/*
 * file.d.h.c
 * Copyright (C) 2008-2010, 2013, 2014, 2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include <file-defs.h>
#include <file-types.h>
#include <trans.h>

#define data(file, offset) \
    ((char *) (file(file) + 1) + (offset))

#define file(file) \
    ((struct eifile_type *) (file))

int
_libx1f4i0_read_ifile(void *file, void **line, unsigned *size, unsigned call,
		      int fill, struct trans_type *trans_data)
{
    int status;

    do {
	int done = OVER_CLASS, (*pull)(void *, void *, unsigned);
	unsigned fast, miss;
	void *data, *sd;

	status = trans_data->link(trans_data->data, &data, call + 1);
	if (status) {
	    status = LINK_CLASS;
	    if (1) {
		break;
	    }
	}

	miss = file(file)->size;
	if (miss) {
	    unsigned ever;

	    ever = file(file)->ever;

	    if (miss < call) {
		memmove(file(file) + 1, data(file, ever), miss);
		file(file)->ever = 0;
	    } else {
		char *text;

		memcpy(data, data(file, ever), call);
		text = data;
		text[call] = 0;

		*size = call;

		file(file)->ever = ever + call;
		file(file)->size = miss - call;

		*line = data;

		break;
	    }
	}

	fast = EVER_MAX;

	pull = file(file)->sf->pull;

	sd = &file(file)->sd;

	while (1) {
	    if (miss < EVER_MAX) {
		status = pull(sd, data(file, miss), EVER_MAX - miss);
	    } else {
		status = pull(sd, (char *) data + fast, call - fast);
	    }
	    if (status == -1) {
		if (errno == EINTR) {
		    continue;
		} else {
		    if (errno == EAGAIN || errno == EWOULDBLOCK) {
			done = NONE_CLASS;

			status = 0;
		    } else {
			status = READ_CLASS;

			trans_data->free(trans_data->data, data);

			if (1) {
			    break;
			}
		    }
		}
	    }

	    if (status) {
		unsigned ever;

		ever = status;

		if (miss < EVER_MAX) {
		    miss += ever;
		    if (miss < call) {
		    } else {
			char *text;

			text = data;

			memcpy(text, file(file) + 1, call);

			*line = data;
			*size = call;

			text[call] = 0;

			file(file)->ever = call;
			file(file)->size = miss - call;

			status = 0;

			break;
		    }
		} else {
		    fast += ever;
		    if (fast < call) {
		    } else {
			char *text;

			text = data;

			memcpy(text, file(file) + 1, EVER_MAX);

			*line = data;
			*size = fast;

			text[fast] = 0;

			file(file)->size = 0;

			status = 0;

			break;
		    }
		}
	    } else {
		if (miss && !fill) {
		    char *text;

		    text = data;

		    memcpy(text, file(file) + 1, miss);

		    if (miss < EVER_MAX) {
			fast = miss;
		    }

		    if (fast ^ call) {
			status = trans_data->mode
			    (trans_data->data, &data, fast + 1);
			if (status) {
			    trans_data->free(trans_data->data, data);

			    status = MODE_CLASS;

			    break;
			} else {
			    text = data;
			}
		    }

		    *line = data;
		    *size = fast;

		    text[fast] = 0;

		    file(file)->size = 0;
		} else {
		    file(file)->ever = 0;
		    file(file)->size = miss;

		    status = trans_data->free(trans_data->data, data);
		    if (status) {
			status = FREE_CLASS;
		    } else {
			status = done;

			if (EVER_MAX < fast) {
			    file(file)->sf->seek
				(sd, EVER_MAX - (int) fast, SEEK_CUR);
			}
		    }
		}

		break;
	    }
	}
    } while (0);

    return status;
}
