/*
 * lxdash-a.l.c
 * Copyright (C) 2008-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <d_fset.h>
#include <e4.h>
#include <lxcall.h>
#include <lxlead.h>
#include <lxdash-defs.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

#define dxdash(line) \
    ((struct dxdash_type *) (line))
#define lxdash(line) \
    ((struct lxdash_type *) (line))

#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

typedef struct text_type {
    int delete;
    struct excase_type *excase_data, *excase_lock;
    void *lxdash;
} text_type;

static int flat_node(void *, void *);

static int
flat_node(void *text, void *node)
{
    struct effect_type *effect_data;
    unsigned link;

    effect_data = node;

    link = effect_data->link;
    if (~link) {
	int (*free) (void *, void *, struct excase_type **,
		     const struct excase_type *);
	struct lxdash_type *lxdash_data;
	struct lxtext_type *lxtext_link;

	lxdash_data = ((struct text_type *) text)->lxdash;

	lxtext_link = ((struct lxtext_type *) lxdash_data->link_f.text) + link;

	if (lxtext_link->lxtype.flags & X1f4_LX_PICK_ACCESS) {
	    free = lxtext_link->lxtype.pick;
	} else {
	    free = lxtext_link->lxtype.free;
	}
	if (free) {
	    int delete;

	    delete = free
		(lxtext_link->lxtype.context, I_USER(&effect_data->fxdata),
		 &((struct text_type *) text)->excase_data,
		 ((struct text_type *) text)->excase_lock);
	    if (delete) {
		if (((struct text_type *) text)->delete) {
		} else {
		    ((struct text_type *) text)->delete = delete;
		}
	    }
	}
    }

    return 0;
}


int
_libx1f4i0_lxdash_null_line(void *lxdash,
			    struct excase_type **excase, void *dxdash)
{
    int status = 0;
    unsigned size;

    size = dxdash(dxdash)->size;
    if (size) {
	int excess;
	struct excase_type *excase_data;
	void *line;

	if (excase) {
	    excase_data = *excase;
	} else {
	    excase_data = NULL;
	}

	line = dxdash(dxdash)->dash;

	if (1) {
	    struct text_type text;

	    text.delete = 0;
	    text.excase_data = excase_data;
	    text.excase_lock = lock_excase(dxdash);
	    text.lxdash = lxdash;

	    _libx1f4i0_lime_lxfset(line, &text, flat_node);

	    excase_data = text.excase_data;

	    status = text.delete;

	    dxdash(dxdash)->size = 0;
	}

	if (1) {
	    excess = _libx1f4i0_high_lxfset(line);
	    if (excess) {
		status = _libx1f4i0_lxdash_stat_line(lxdash, status, excess);
	    }
	}

	if (excase) {
	    *excase = excase_data;
	} else {
	    if (excase_data) {
		excess = x1f4_fail_lxlead(excase_data);
		if (excess) {
		    status = _libx1f4i0_lxcall_land_slip(excess, status);
		}
	    }
	}
    }

    return status;
}
