/*
 * lxline-b.e.c
 * Copyright (C) 2008-2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxline-config.h>

#include <e4.h>
#include <l_note.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#define I_BILL(t)			(*((X1f4_E4_C_BILL *) (t)))
#define I_LINE(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

#define l_BILL(e, output) \
    {									      \
	X1f4_E4_C_BILL *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define byte(miss)			((unsigned char *) (miss))

#if __HAVE_SHUT_SFNOTE__
static int shut_note(void *, void *);
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
static int
shut_note(void *text, void *data)
{
    int status = 0;
    unsigned type;

    type = byte(data)[0] << 030 | byte(data)[1] << 020
	| byte(data)[2] << 010 | byte(data)[3];
    if (type != X1f4_E4_BILL) {
	int *delete;

	delete = *((void **) text + 3);
	status = _libx1f4i0_lxline_stat_type
	    (*(void **) text, I_TEXT((*((void ***) text + 2))[2]), type,
	     X1f4_E4_BILL);
	*delete = status;
    } else {
	X1f4_E4_C_BILL fast;

	fast = I_BILL(byte(data) + 8);

	l_BILL(fast, *((void **) text + 1));
	l_BILL(fast, **((void ***) text + 2));
    }

    return status;
}
#endif				/* __HAVE_SHUT_SFNOTE__ */


int
_libx1f4i0_lxline_e_class_b(void *context, void *output, void **input)
{
    X1f4_E4_C_TEXT fail;
#if __HAVE_SHUT_SFNOTE__
    int excess;
#endif				/* __HAVE_SHUT_SFNOTE__ */
    int status;
    struct dxline_type *dxline_data;
    struct lxline_type *lxline_data;
    struct lxtext_type *lxtext_data;
#if __HAVE_SHUT_SFNOTE__
#else
    void *data;
#endif				/* __HAVE_SHUT_SFNOTE__ */
#if __HAVE_SHUT_SFNOTE__
    void *text[4];
#endif				/* __HAVE_SHUT_SFNOTE__ */

    lxtext_data = context;

    lxline_data = lxtext_data->text;

    dxline_data = I_LINE(input[1]);

    fail = I_TEXT(input[2]);

#if __HAVE_SHUT_SFNOTE__
    text[0] = lxline_data;
    text[1] = output;
    text[2] = input;
    text[3] = &excess;
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
    status = _libx1f4i0_shut_sfnote(dxline_data->line, fail, text, shut_note);
    if (status) {
	if (status == X1f4_SFNOTE_MISS_CLASS) {
	    status = _libx1f4i0_lxline_stat_miss(lxline_data, fail);
	} else {
	    if (status == X1f4_SFNOTE_CALL_ERROR) {
		status = excess;
	    } else {
		status = _libx1f4i0_lxline_stat_line(lxline_data, 0, status);
	    }
	}
    } else {
	dxline_data->size--;
    }
#else
    status = _libx1f4i0_find_sfnote(dxline_data->line, fail, &data);
    if (status) {
	unsigned type;

	type = byte(data)[0] << 030 | byte(data)[1] << 020
	    | byte(data)[2] << 010 | byte(data)[3];
	if (type != X1f4_E4_BILL) {
	    status = _libx1f4i0_lxline_stat_type
		(lxline_data, fail, type, X1f4_E4_BILL);
	} else {
	    X1f4_E4_C_BILL fast;

	    fast = I_BILL(byte(data) + 8);

	    status = _libx1f4i0_pass_sfnote(dxline_data->line, fail);
	    if (status) {
		status = _libx1f4i0_lxline_stat_line(lxline_data, 0, status);
	    } else {
		dxline_data->size--;

		if (0) {
		} else {
		    l_BILL(fast, input[0]);
		    l_BILL(fast, output);
		}
	    }
	}
    } else {
	status = _libx1f4i0_lxline_stat_miss(lxline_data, fail);
    }
#endif				/* __HAVE_SHUT_SFNOTE__ */

    return status;
}
