/*
 * lxline-v.o.c
 * Copyright (C) 2008-2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <l_note.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#define I_LINE(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define byte(miss)			((unsigned char *) (miss))

#if __HAVE_SHUT_SFNOTE__
static int shut_note(void *, void *);
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
static int
shut_note(void *text, void *data)
{
    int status = 0;
    struct lxtext_type *lxtext_data;
    unsigned type;

    lxtext_data = *(void **) text;

    type = effect_type(data);
    if (type != lxtext_data->lxtype.code) {
	int *delete;

	delete = *((void **) text + 3);
	status = 1;
	*delete = 0;
    } else {
	void *depart;

	depart = **((void ***) text + 2);

	status = lxtext_data->lxtype.push
	    (lxtext_data->lxtype.context, depart, I_USER(byte(data) + 8),
	     lock_excase(*((void **) text + 4)));
	if (status) {
	    int *delete;

	    delete = *((void **) text + 3);
	    *delete = status;
	} else {
	    byte(data)[4] = ~0 >> 030;
	    byte(data)[5] = ~0 >> 020;
	    byte(data)[6] = ~0 >> 010;
	    byte(data)[7] = ~0 >> 000;
	}
    }

    return status;
}
#endif				/* __HAVE_SHUT_SFNOTE__ */


int
_libx1f4i0_lxline_o_class_v(void *context, void *output, void **input)
{
    X1f4_E4_C_TEXT fail;
#if __HAVE_SHUT_SFNOTE__
    int excess;
#endif				/* __HAVE_SHUT_SFNOTE__ */
    int status;
    struct dxline_type *dxline_data;
    struct lxtext_type *lxtext_data;
#if __HAVE_SHUT_SFNOTE__
#else
    void *data;
#endif				/* __HAVE_SHUT_SFNOTE__ */
#if __HAVE_SHUT_SFNOTE__
    void *text[5];
#endif				/* __HAVE_SHUT_SFNOTE__ */

    lxtext_data = context;

    dxline_data = I_LINE(input[1]);

    fail = I_TEXT(input[2]);

#if __HAVE_SHUT_SFNOTE__
    text[0] = lxtext_data;
    text[2] = input;
    text[3] = &excess;
    text[4] = dxline_data;
#endif				/* __HAVE_SHUT_SFNOTE__ */

#if __HAVE_SHUT_SFNOTE__
    status = _libx1f4i0_shut_sfnote(dxline_data->line, fail, text, shut_note);
    if (status) {
	if (status == X1f4_SFNOTE_MISS_CLASS) {
	    status = 0;

            l_MODE(0, output);
	} else {
	    if (status == X1f4_SFNOTE_CALL_ERROR) {
		status = excess;
		if (status) {
		} else {
		    l_MODE(0, output);
		}
	    } else {
		status = _libx1f4i0_lxline_stat_line
		    (lxtext_data->text, 0, status);
	    }
	}
    } else {
	dxline_data->size--;

	if (1) {
	    l_MODE(1, output);
	}
    }
#else
    status = _libx1f4i0_find_sfnote(dxline_data->line, fail, &data);
    if (status) {
	unsigned type;

	type = effect_type(data);
	if (type != lxtext_data->lxtype.code) {
	    l_MODE(0, output);
	    if (1) {
		status = 0;
	    }
	} else {
	    status = lxtext_data->lxtype.push
		(lxtext_data->lxtype.context, input[0],
		 I_USER(byte(data) + 8), lock_excase(dxline_data));
	    if (status) {
	    } else {
		byte(data)[4] = ~0 >> 030;
		byte(data)[5] = ~0 >> 020;
		byte(data)[6] = ~0 >> 010;
		byte(data)[7] = ~0 >> 000;

		status = _libx1f4i0_pass_sfnote(dxline_data->line, fail);
		if (status) {
		    status = _libx1f4i0_lxline_stat_line
			(lxtext_data->text, 0, status);
		} else {
		    dxline_data->size--;

		    if (1) {
			l_MODE(1, output);
		    }
		}
	    }
	}
    } else {
	l_MODE(0, output);
    }
#endif				/* __HAVE_SHUT_SFNOTE__ */

    return status;
}
