/*
 * lxlist-a.0.c
 * Copyright (C) 2008-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * the U/V distinction is fluid, made sometimes on -lead- (the proper 
 * criterion), sometimes on -push- or whatever convenient.
 */

#include <lxlist-config.h>

#include <stddef.h>
#include <string.h>

#include <af.h>
#include <e4-m.0.h>
#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-defs.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>
#include <lxnext.h>
#include <mxdeck.h>
#include <srand.h>

#define MAKE_SINGLE(a, b)		a

#define may_include(lxtype)		0

#define l_e_type_post			3

#define l_g_type_post			3

#define l_i_type_post			3

#define lbe_type_post			2

#define lbg_type_post			2

#define lfe_type_post			2

#define lfg_type_post			2

#define true(e)				1

#define l________size \
    sizeof(struct x1f4_linetext_type)

#define part(screen) \
    ((struct part_type *) (screen))
#define screen(screen) \
    (((struct part_type *) (screen))->screen_data)

#define fm_size(none) \
    sizeof(struct lxmiss_type)

#define fb_size()			2

typedef struct part_type {
    const struct screen_type *screen_data;
    struct x1f4_ftaa_type *ftaa_data;
    struct x1f4_ftca_type *ftca_data;
} part_type;

extern const struct x1f4_eelookup_type _x1f4_type_lookup;

static int init_list(struct lxlist_type *, unsigned,
		     const struct screen_type *);
static int line_data(struct lxlist_type *, unsigned, const struct part_type *,
		     void *, void *, void **, void **);
static int line_list(struct lxlist_type *, unsigned,
		     const struct screen_type *);
static int line_plus(struct lxlist_type *);
static int line_sail(struct lxlist_type *, const struct screen_type *,
		     struct x1f4_linetext_type **, struct lxtext_type *,
		     const struct lxtype_type *, void **, void **);
static int line_text(struct lxlist_type *, unsigned, const struct part_type *,
		     void *, void **, void **);
static int link_list(struct lxlist_type *, unsigned,
		     const struct screen_type *);
static int pick_miss(unsigned *, unsigned *, unsigned *, unsigned,
		     const struct screen_type *);

static void bo_slmy00(u_case_args_____0);
static void born_lmlm(u_case_args_____0);
static void borne_lmc(u_case_args_____0);
static void borne_lmm(u_case_args_____0);
static void borne_lmn(v_case_args_____0);
static void borneo__l(u_case_args_____0);
static void borneo_cl(u_case_args_____0);
static void borneo_l_(u_case_args_____0);
static void borneo_lc(u_case_args_____0);
static void borneo_ll(u_case_args_____0);
static void borneo_lm(u_case_args_____0);
static void borneo_ln(v_case_args_____0);
static void fine_pmix(v_case_args_____0);
static void line_emix(v_case_args_____0);
static void line_epix(v_case_args_____0);
static void line_esix(v_case_args_____0);
static void line_gmix(v_case_args_____0);
static void line_gpix(v_case_args_____0);
static void line_gsix(v_case_args_____0);
static void line_imix(v_case_args_____0);
static void line_nfix(v_case_args_____0);
static void line_nset(u_case_args_____0);
static void line_pmix(v_case_args_____0);
static void line_ppix(v_case_args_____0);
static void line_psix(v_case_args_____0);
static void line_qmix(v_case_args_____0);
static void line_qpix(v_case_args_____0);
static void line_qsix(v_case_args_____0);
static void line_rmix(v_case_args_____0);
static void line_smap(v_case_args_____0);
static void line_sset(u_case_args_____0);
static void line_xmix(v_case_args_____0);
static void line_xpix(v_case_args_____0);
static void line_xsix(v_case_args_____0);
static void line_ymix(v_case_args_____0);

static const char fl_queryf[] = "l_query", fl_setf[] = "l_set",
    *const side_f[] = {
/* *INDENT-OFF* */
    fl_setf,
    fl_queryf
/* *INDENT-ON* */
};
static int (*const rate_emix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_e_class_b,
    _libx1f4i0_lxlist_e_class_m,
    _libx1f4i0_lxlist_e_class_r,
    _libx1f4i0_lxlist_e_class_t,
    _libx1f4i0_lxlist_e_class_u,
    _libx1f4i0_lxlist_e_class_v
/* *INDENT-ON* */
}, (*const rate_epix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_e0class_b,
    _libx1f4i0_lxlist_e0class_m,
    _libx1f4i0_lxlist_e0class_r,
    _libx1f4i0_lxlist_e0class_t,
    _libx1f4i0_lxlist_ebclass_u,
    _libx1f4i0_lxlist_ebclass_v
/* *INDENT-ON* */
}, (*const rate_esix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_e0class_b,
    _libx1f4i0_lxlist_e0class_m,
    _libx1f4i0_lxlist_e0class_r,
    _libx1f4i0_lxlist_e0class_t,
    _libx1f4i0_lxlist_efclass_u,
    _libx1f4i0_lxlist_efclass_v
/* *INDENT-ON* */
}, (*const rate_gmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_g_class_b,
    _libx1f4i0_lxlist_g_class_m,
    _libx1f4i0_lxlist_g_class_r,
    _libx1f4i0_lxlist_g_class_t,
    _libx1f4i0_lxlist_g_class_u,
    _libx1f4i0_lxlist_g_class_v
/* *INDENT-ON* */
}, (*const rate_gpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_g0class_b,
    _libx1f4i0_lxlist_g0class_m,
    _libx1f4i0_lxlist_g0class_r,
    _libx1f4i0_lxlist_g0class_t,
    _libx1f4i0_lxlist_gbclass_u,
    _libx1f4i0_lxlist_gbclass_v
/* *INDENT-ON* */
}, (*const rate_gsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_g0class_b,
    _libx1f4i0_lxlist_g0class_m,
    _libx1f4i0_lxlist_g0class_r,
    _libx1f4i0_lxlist_g0class_t,
    _libx1f4i0_lxlist_gfclass_u,
    _libx1f4i0_lxlist_gfclass_v
/* *INDENT-ON* */
}, (*const rate_imix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_i_class_b,
    _libx1f4i0_lxlist_i_class_m,
    _libx1f4i0_lxlist_i_class_r
/* *INDENT-ON* */
}, (*const rate_pmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_p_class_b,
    _libx1f4i0_lxlist_p_class_m,
    _libx1f4i0_lxlist_p_class_r,
    _libx1f4i0_lxlist_p_class_t,
    _libx1f4i0_lxlist_p_class_u,
    _libx1f4i0_lxlist_p_class_v
/* *INDENT-ON* */
}, (*const rate_qmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_q_class_b,
    _libx1f4i0_lxlist_q_class_m,
    _libx1f4i0_lxlist_q_class_r,
    _libx1f4i0_lxlist_q_class_t,
    _libx1f4i0_lxlist_q_class_u,
    _libx1f4i0_lxlist_q_class_v
/* *INDENT-ON* */
}, (*const rate_qpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_q0class_b,
    _libx1f4i0_lxlist_q0class_m,
    _libx1f4i0_lxlist_q0class_r,
    _libx1f4i0_lxlist_q0class_t,
    _libx1f4i0_lxlist_qbclass_u,
    _libx1f4i0_lxlist_qbclass_v
/* *INDENT-ON* */
}, (*const rate_qsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_q0class_b,
    _libx1f4i0_lxlist_q0class_m,
    _libx1f4i0_lxlist_q0class_r,
    _libx1f4i0_lxlist_q0class_t,
    _libx1f4i0_lxlist_qfclass_u,
    _libx1f4i0_lxlist_qfclass_v
/* *INDENT-ON* */
}, (*const rate_rmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_r_class_b,
    _libx1f4i0_lxlist_r_class_m,
    _libx1f4i0_lxlist_r_class_r,
    _libx1f4i0_lxlist_r_class_t,
    _libx1f4i0_lxlist_r_class_u,
    _libx1f4i0_lxlist_r_class_v
/* *INDENT-ON* */
}, (*const rate_xmix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_x_class_b,
    _libx1f4i0_lxlist_x_class_m,
    _libx1f4i0_lxlist_x_class_r,
    _libx1f4i0_lxlist_x_class_t,
    _libx1f4i0_lxlist_x_class_u,
    _libx1f4i0_lxlist_x_class_v
/* *INDENT-ON* */
}, (*const rate_xpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_x0class_b,
    _libx1f4i0_lxlist_x0class_m,
    _libx1f4i0_lxlist_x0class_r,
    _libx1f4i0_lxlist_x0class_t,
    _libx1f4i0_lxlist_xbclass_u,
    _libx1f4i0_lxlist_xbclass_v
/* *INDENT-ON* */
}, (*const rate_xsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_x0class_b,
    _libx1f4i0_lxlist_x0class_m,
    _libx1f4i0_lxlist_x0class_r,
    _libx1f4i0_lxlist_x0class_t,
    _libx1f4i0_lxlist_xfclass_u,
    _libx1f4i0_lxlist_xfclass_v
/* *INDENT-ON* */
}, (*const rate_ymix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxlist_y_class_b,
    _libx1f4i0_lxlist_y_class_m,
    _libx1f4i0_lxlist_y_class_r
/* *INDENT-ON* */
};
static const struct line_type auto_line[] = {
/* *INDENT-OFF* */
#define l_assemble_count		0
    {	{	"l_assemble",
		_libx1f4i0_lxlist_a_centric,	0,
		NULL,				l_assemble_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						10		},
	borneo_l_,							},
#define l_assemble_reach		(0 + l_assemble_count)
#define l_bill_count			2
    {	{	"l_bill",
		_libx1f4i0_lxlist_b1centric,	X1f4_E4_VOID,
		NULL,				l_bill_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						6		},
	borneo_lm,							},
#define l_bill_reach			(l_assemble_reach + l_bill_count)
#define l_collect_count			2
    {	{	"l_collect",
		_libx1f4i0_lxlist_c1centric,	X1f4_E4_VOID,
		NULL,				l_collect_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						9		},
	borneo_lm,							},
#define l_collect_reach			(l_bill_reach + l_collect_count)
#define l_effect_count			0
    {	{	"l_effect",
		_libx1f4i0_lxlist_e_centric,	0,
		NULL,				l_effect_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_SIDE_LIST
		| X1f4_E4_TEXT_LINK,
						8		},
	borneo_l_,							},
#define l_effect_reach			(l_collect_reach + l_effect_count)
#define l_extract_count			3
    {	{	"l_extract",
		_libx1f4i0_lxlist_y_derived,	X1f4_E4_VOID,
		NULL,				l_extract_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	bo_slmy00,							},
#define l_extract_reach \
    (l_effect_reach + (l_extract_count << 1))
#define l_get_count			3
    {	{	"l_get",
		_libx1f4i0_lxlist_g_derived,	X1f4_E4_VOID,
		NULL,				l_get_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						5		},
	bo_slmy00,							},
#define l_get_reach			(l_extract_reach + (l_get_count << 1))
#define l_jack_count			3
    {	{	"l_jack",
		_libx1f4i0_lxlist_j_derived,	X1f4_E4_MODE,
		NULL,				l_jack_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	bo_slmy00,							},
#define l_jack_reach			(l_get_reach + (l_jack_count << 1))
#define l_new_count			1
    {	{	"l_new",
		_libx1f4i0_lxlist_n_forward,	X1f4_E4_VOID,
		NULL,				l_new_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,
						5		},
	line_nset,							},
#define l_new_reach			(l_jack_reach + (l_new_count << 1))
#define l_over_count			3
    {	{	"l_over",
		_libx1f4i0_lxlist_o_derived,	X1f4_E4_MODE,
		NULL,				l_over_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	bo_slmy00,							},
#define l_over_reach			(l_new_reach + (l_over_count << 1))
#define l_xcall_count			2
    {	{	"l_xcall",
		_libx1f4i0_lxlist_x_pending,	X1f4_E4_CASE,
		NULL,				l_xcall_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS
		| X1f4_E4_SIDE_LIST | X1f4_E4_TEXT_LINK
		| X1f4_E4_WALK_LINK,
						7		},
	borneo_cl,							},
#define l_xcall_reach			(l_over_reach + (l_xcall_count << 1))
#define l_yank_count			3
    {	{	"l_yank",
		_libx1f4i0_lxlist_y_derived,	X1f4_E4_VOID,
		NULL,				l_yank_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						6		},
	bo_slmy00,							}
#define l_yank_reach			(l_xcall_reach + (l_yank_count << 1))
/* *INDENT-ON* */
#define auto_line_count			l_yank_reach
}, ever_line[] = {
/* *INDENT-OFF* */
#define l_set_count			2
    {	{	fl_setf,
		_libx1f4i0_lxlist_s_forward,	0,
		NULL,				l_set_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,
						5		},
	line_sset,							}
#define l_set_reach			(0 + (l_set_count << 1))
/* *INDENT-ON* */
#define ever_line_count			l_set_reach
}, fail_line[] = {
/* *INDENT-OFF* */
#define l_bind_count			3
    {	{	"l_bind",
		_libx1f4i0_lxlist_b_pending,	X1f4_E4_VOID,
		NULL,				l_bind_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_lmc,							},
#define l_bind_reach			(0 + l_bind_count)
#define l_link_count			3
    {	{	"l_link",
		_libx1f4i0_lxlist_l_pending,	X1f4_E4_VOID,
		NULL,				l_link_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_lmc,							},
#define l_link_reach			(l_bind_reach + l_link_count)
#define l_pick_count			2
    {	{	"l_pick",
		_libx1f4i0_lxlist_p_derived,	X1f4_E4_CASE,
		NULL,				l_pick_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lm,							},
#define l_pick_reach			(l_link_reach + l_pick_count)
#define l_push_count			3
    {	{	"l_push",
		_libx1f4i0_lxlist_p_forward,	X1f4_E4_VOID,
		NULL,				l_push_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_lmc,							},
#define l_push_reach			(l_pick_reach + l_push_count)
#define l_query_count			2
    {	{	fl_queryf,
		_libx1f4i0_lxlist_q_forward,	X1f4_E4_CASE,
		NULL,				l_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lm,							},
#define l_query_reach			(l_push_reach + l_query_count)
#define l_replace_count			3
    {	{	"l_replace",
		_libx1f4i0_lxlist_r_logique,	X1f4_E4_VOID,
		NULL,				l_replace_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	borne_lmc,							}
#define l_replace_reach			(l_query_reach + l_replace_count)
/* *INDENT-ON* */
#define fail_line_count			l_replace_reach
}, gray_line[] = {
/* *INDENT-OFF* */
#define l_short_count			1
    {	{	"l_short",
		_libx1f4i0_lxlist_a_keeping,	X1f4_E4_VOID,
		NULL,				l_short_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							}
#define l_short_reach			(0 + l_short_count)
/* *INDENT-ON* */
#define gray_line_count			l_short_reach
}, head_line[] = {
/* *INDENT-OFF* */
#define l_front_count			1
    {	{	"l_front",
		_libx1f4i0_lxlist_c_keeping,	X1f4_E4_CASE,
		NULL,				l_front_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define l_front_reach			(0 + l_front_count)
#define l_insert_count			2
    {	{	"l_insert",
		_libx1f4i0_lxlist_b_keeping,	X1f4_E4_VOID,
		NULL,				l_insert_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lc,							},
#define l_insert_reach			(l_front_reach + l_insert_count)
#define lf_link_count			2
    {	{	"lf_link",
		_libx1f4i0_lxlist_d_keeping,	X1f4_E4_VOID,
		NULL,				lf_link_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lc,							},
#define lf_link_reach			(l_insert_reach + lf_link_count)
#define lf_pick_count			1
    {	{	"lf_pick",
		_libx1f4i0_lxlist_e_keeping,	X1f4_E4_CASE,
		NULL,				lf_pick_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define lf_pick_reach			(lf_link_reach + lf_pick_count)
#define lf_push_count			2
    {	{	"lf_push",
		_libx1f4i0_lxlist_b_keeping,	X1f4_E4_VOID,
		NULL,				lf_push_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lc,							},
#define lf_push_reach			(lf_pick_reach + lf_push_count)
#define lf_query_count			1
    {	{	"lf_query",
		_libx1f4i0_lxlist_c_keeping,	X1f4_E4_CASE,
		NULL,				lf_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo__l,							}
#define lf_query_reach			(lf_push_reach + lf_query_count)
/* *INDENT-ON* */
#define head_line_count			lf_query_reach
}, slip_line[] = {
/* *INDENT-OFF* */
#define l_cast_count			2
    {	{	"l_cast",
		_libx1f4i0_lxlist_c_nearest,	X1f4_E4_VOID,
		NULL,				l_cast_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_ll,							},
#define l_cast_reach			(0 + l_cast_count)
#define l_clear_count			1
    {	{	"l_clear",
		_libx1f4i0_lxlist_z_logique,	X1f4_E4_VOID,
		NULL,				l_clear_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define l_clear_reach			(l_cast_reach + l_clear_count)
#define l_copy_count			2
    {	{	"l_copy",
		_libx1f4i0_lxlist_c_logique,	X1f4_E4_VOID,
		NULL,				l_copy_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_ll,							},
#define l_copy_reach			(l_clear_reach + l_copy_count)
#define l_delete_count			2
    {	{	"l_delete",
		_libx1f4i0_lxlist_d_logique,	X1f4_E4_VOID,
		NULL,				l_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lm,							},
#define l_delete_reach			(l_copy_reach + l_delete_count)
#define l_down_count			2
    {	{	"l_down",
		_libx1f4i0_lxlist_d_forward,	X1f4_E4_VOID,
		NULL,				l_down_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lm,							},
#define l_down_reach			(l_delete_reach + l_down_count)
#define l_edit_count			3
    {	{	"l_edit",
		_libx1f4i0_lxlist_e_forward,	X1f4_E4_VOID,
		NULL,				l_edit_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_lmm,							},
#define l_edit_reach			(l_down_reach + l_edit_count)
#define l_erase_count			3
    {	{	"l_erase",
		_libx1f4i0_lxlist_e_pending,	X1f4_E4_VOID,
		NULL,				l_erase_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borne_lmm,							},
#define l_erase_reach			(l_edit_reach + l_erase_count)
#define l_id_count			2
    {	{	"l_id",
		_libx1f4i0_lxlist_i_pending,	X1f4_E4_MODE,
		NULL,				l_id_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	borneo_lm,							},
#define l_id_reach			(l_erase_reach + l_id_count)
#define l_length_count			1
    {	{	"l_length",
		_libx1f4i0_lxlist_l_logique,	X1f4_E4_MODE,
		NULL,				l_length_count,
		0,
						8		},
	borneo__l,							},
#define l_length_reach			(l_id_reach + l_length_count)
#define l_post_count			4
    {	{	"l_post",
		_libx1f4i0_lxlist_k_logique,	X1f4_E4_VOID,
		NULL,				l_post_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	born_lmlm,							},
#define l_post_reach			(l_length_reach + l_post_count)
#define l_move_count			3
    {	{	"l_move",
		_libx1f4i0_lxlist_m_logique,	X1f4_E4_VOID,
		NULL,				l_move_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_lmm,							},
#define l_move_reach			(l_post_reach + l_move_count)
#define l_offset_count			2
    {	{	"l_offset",
		_libx1f4i0_lxlist_o_logique,	X1f4_E4_VOID,
		NULL,				l_offset_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lm,							},
#define l_offset_reach			(l_move_reach + l_offset_count)
#define l_rand_count			1
    {	{	"l_rand",
		_libx1f4i0_lxlist_r_derived,	X1f4_E4_VOID,
		NULL,				l_rand_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo__l,							},
#define l_rand_reach			(l_offset_reach + l_rand_count)
#define l_reverse_count			1
    {	{	"l_reverse",
		_libx1f4i0_lxlist_r_forward,	X1f4_E4_VOID,
		NULL,				l_reverse_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	borneo__l,							},
#define l_reverse_reach			(l_rand_reach + l_reverse_count)
#define l_spin_count			3
    {	{	"l_spin",
		_libx1f4i0_lxlist_s_logique,	X1f4_E4_VOID,
		NULL,				l_spin_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borne_lmm,							},
#define l_spin_reach			(l_reverse_reach + l_spin_count)
#define l_swap_count			2
    {	{	"l_swap",
		_libx1f4i0_lxlist_s_sequent,	X1f4_E4_VOID,
		NULL,				l_swap_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_ll,							},
#define l_swap_reach			(l_spin_reach + l_swap_count)
#define l_trade_count			4
    {	{	"l_trade",
		_libx1f4i0_lxlist_t_logique,	X1f4_E4_VOID,
		NULL,				l_trade_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	born_lmlm,							},
#define l_trade_reach			(l_swap_reach + l_trade_count)
#define l_type_count			2
    {	{	"l_type",
		_libx1f4i0_lxlist_t_pending,	X1f4_E4_TEXT,
		NULL,				l_type_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo_lm,							},
#define l_type_reach			(l_trade_reach + l_type_count)
#define l_up_count			2
    {	{	"l_up",
		_libx1f4i0_lxlist_u_forward,	X1f4_E4_VOID,
		NULL,				l_up_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	borneo_lm,							}
#define l_up_reach			(l_type_reach + l_up_count)
/* *INDENT-ON* */
#define slip_line_count			l_up_reach
}, tail_line[] = {
/* *INDENT-OFF* */
#define l_append_count			2
    {	{	"l_append",
		_libx1f4i0_lxlist_b_keeping,	X1f4_E4_VOID,
		NULL,				l_append_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo_lc,							},
#define l_append_reach			(0 + l_append_count)
#define l_back_count			1
    {	{	"l_back",
		_libx1f4i0_lxlist_c_keeping,	X1f4_E4_CASE,
		NULL,				l_back_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	borneo__l,							},
#define l_back_reach			(l_append_reach + l_back_count)
#define lb_link_count			2
    {	{	"lb_link",
		_libx1f4i0_lxlist_d_keeping,	X1f4_E4_VOID,
		NULL,				lb_link_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lc,							},
#define lb_link_reach			(l_back_reach + lb_link_count)
#define lb_pick_count			1
    {	{	"lb_pick",
		_libx1f4i0_lxlist_e_keeping,	X1f4_E4_CASE,
		NULL,				lb_pick_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							},
#define lb_pick_reach			(lb_link_reach + lb_pick_count)
#define lb_push_count			2
    {	{	"lb_push",
		_libx1f4i0_lxlist_b_keeping,	X1f4_E4_VOID,
		NULL,				lb_push_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo_lc,							},
#define lb_push_reach			(lb_pick_reach + lb_push_count)
#define lb_query_count			1
    {	{	"lb_query",
		_libx1f4i0_lxlist_c_keeping,	X1f4_E4_CASE,
		NULL,				lb_query_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						8		},
	borneo__l,							}
#define lb_query_reach			(lb_push_reach + lb_query_count)
/* *INDENT-ON* */
#define tail_line_count			lb_push_reach
}, used_line[] = {
/* *INDENT-OFF* */
#define l_class_count			1
    {	{	"l_class",
		_libx1f4i0_lxlist_a_keeping,	X1f4_E4_VOID,
		NULL,				l_class_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	borneo__l,							}
#define l_class_reach			l_class_count
/* *INDENT-ON* */
#define used_line_count			l_class_reach
};
static const struct link_type cast_line[] = {
/* *INDENT-OFF* */
#define l_q__class			4
#define l_q__count			2
    {	{	"l_q_",
		NULL,				0,
		NULL,				l_q__count,
		X1f4_E4_TEXT_LINK,		l_q__class	},
	line_qmix,							},
#define l_q__reach			(0 + l_q__count)
#define l_q__trans			(0 + l_q__class)
#define lb_q__class			5
#define lb_q__count			1
    {	{	"lb_q_",
		NULL,				0,
		NULL,				lb_q__count,
		X1f4_E4_TEXT_LINK,		lb_q__class	},
	line_qpix,							},
#define lb_q__reach			(l_q__reach + lb_q__count)
#define lb_q__trans			(l_q__trans + lb_q__class)
#define lf_q__class			5
#define lf_q__count			1
    {	{	"lf_q_",
		NULL,				0,
		NULL,				lf_q__count,
		X1f4_E4_TEXT_LINK,		lf_q__class	},
	line_qsix,							}
#define lf_q__reach			(lb_q__reach + lf_q__count)
#define lf_q__trans			(lb_q__trans + lf_q__class)
/* *INDENT-ON* */
#define cast_line_class			lf_q__trans
#define cast_line_count			lf_q__reach
#define cast_line_shift \
    sizeof(cast_line) / sizeof(struct link_type)
}, fast_line[] = {
/* *INDENT-OFF* */
#define l_e__class			4
#define l_e__count			3
    {	{	"l_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				l_e__count,
		0,				l_e__class	},
	line_emix,							},
#define l_e__reach			(0 + l_e__count)
#define l_e__trans			(0 + l_e__class)
#define l_g__class			4
#define l_g__count			3
    {	{	"l_g_",
		NULL,				0,
		NULL,				l_g__count,
		0,				l_g__class	},
	line_gmix,							},
#define l_g__reach			(l_e__reach + l_g__count)
#define l_g__trans			(l_e__trans + l_g__class)
#define l_p__class			4
#define l_p__count			3
    {	{	"l_p_",
		NULL,				X1f4_E4_VOID,
		NULL,				l_p__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						l_p__class	},
	line_pmix,							},
#define l_p__reach			(l_g__reach + l_p__count)
#define l_p__trans			(l_g__trans + l_p__class)
#define l_r__class			4
#define l_r__count			3
    {	{	"l_r_",
		NULL,				X1f4_E4_VOID,
		NULL,				l_r__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						l_r__class	},
	line_rmix,							},
#define l_r__reach			(l_p__reach + l_r__count)
#define l_r__trans			(l_p__trans + l_r__class)
#define l_s__class			4
#define l_s__count			2
    {	{	"l_s_",
		_libx1f4i0_lxlist_s_pending,	X1f4_E4_MODE,
		NULL,				l_s__count,
		X1f4_E4_TEXT_LINK,		l_s__class	},
	line_smap,							},
#define l_s__reach			(l_r__reach + l_s__count)
#define l_s__trans			(l_r__trans + l_s__class)
#define lb_e__class			5
#define lb_e__count			2
    {	{	"lb_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				lb_e__count,
		0,				lb_e__class	},
	line_epix,							},
#define lb_e__reach			(l_s__reach + lb_e__count)
#define lb_e__trans			(l_s__trans + lb_e__class)
#define lb_g__class			5
#define lb_g__count			2
    {	{	"lb_g_",
		NULL,				0,
		NULL,				lb_g__count,
		0,				lb_g__class	},
	line_gpix,							},
#define lb_g__reach			(lb_e__reach + lb_g__count)
#define lb_g__trans			(lb_e__trans + lb_g__class)
#define lb_p__class			5
#define lb_p__count			2
    {	{	"lb_p_",
		NULL,				X1f4_E4_VOID,
		NULL,				lb_p__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						lb_p__class	},
	line_ppix,							},
#define lb_p__reach			(lb_g__reach + lb_p__count)
#define lb_p__trans			(lb_g__trans + lb_p__class)
#define lf_e__class			5
#define lf_e__count			2
    {	{	"lf_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				lf_e__count,
		0,				lf_e__class	},
	line_esix,							},
#define lf_e__reach			(lb_p__reach + lf_e__count)
#define lf_e__trans			(lb_p__trans + lf_e__class)
#define lf_g__class			5
#define lf_g__count			2
    {	{	"lf_g_",
		NULL,				0,
		NULL,				lf_g__count,
		0,				lf_g__class	},
	line_gsix,							},
#define lf_g__reach			(lf_e__reach + lf_g__count)
#define lf_g__trans			(lf_e__trans + lf_g__class)
#define lf_p__class			5
#define lf_p__count			2
    {	{	"lf_p_",
		NULL,				X1f4_E4_VOID,
		NULL,				lf_p__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						lf_p__class	},
	line_psix,							}
#define lf_p__reach			(lf_g__reach + lf_p__count)
#define lf_p__trans			(lf_g__trans + lf_p__class)
/* *INDENT-ON* */
#define fast_line_class			lf_p__trans
#define fast_line_count			lf_p__reach
#define fast_line_shift \
    sizeof(fast_line) / sizeof(struct link_type)
}, long_line[] = {
/* *INDENT-OFF* */
#define l_b__class			4
#define l_b__count			3
    {	{	"l_b_",
		_libx1f4i0_lxlist_b_forward,	X1f4_E4_VOID,
		NULL,				l_b__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						l_b__class	},
	borne_lmn,							},
#define l_b__reach			(0 + l_b__count)
#define l_b__trans			(0 + l_b__class)
#define l_l__class			4
#define l_l__count			3
    {	{	"l_l_",
		_libx1f4i0_lxlist_l_forward,	X1f4_E4_VOID,
		NULL,				l_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						l_l__class	},
	borne_lmn,							},
#define l_l__reach			(l_b__reach + l_l__count)
#define l_l__trans			(l_b__trans + l_l__class)
#define lb_l__class			5
#define lb_l__count			2
    {	{	"lb_l_",
		_libx1f4i0_lxlist_l_derived,	X1f4_E4_VOID,
		NULL,				lb_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						lb_l__class	},
	borneo_ln,							},
#define lb_l__reach			(l_l__reach + lb_l__count)
#define lb_l__trans			(l_l__trans + lb_l__class)
#define lf_l__class			5
#define lf_l__count			2
    {	{	"lf_l_",
		_libx1f4i0_lxlist_l_sequent,	X1f4_E4_VOID,
		NULL,				lf_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						lf_l__class	},
	borneo_ln,							}
#define lf_l__reach			(lb_l__reach + lf_l__count)
#define lf_l__trans			(lb_l__trans + lf_l__class)
/* *INDENT-ON* */
#define long_line_class			lf_l__trans
#define long_line_count			lf_l__reach
#define long_line_shift \
    sizeof(long_line) / sizeof(struct link_type)
}, near_line[] = {
/* *INDENT-OFF* */
#define l_i__class			4
#define l_i__count			3
    {	{	"l_i_",
		NULL,				0,
		NULL,				l_i__count,
		X1f4_E4_TEXT_LINK,		l_i__class	},
	line_imix,							},
#define l_i__reach			(0 + l_i__count)
#define l_i__trans			(0 + l_i__class)
#define l_pn__class			5
#define l_pn__count			4
    {	{	"l_pn_",
		_libx1f4i0_lxlist_p_nearest,	X1f4_E4_VOID,
		NULL,				l_pn__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						l_pn__class	},
	fine_pmix,							},
#define l_pn__reach			(l_i__reach + l_pn__count)
#define l_pn__trans			(l_i__trans + l_pn__class)
#define l_y__class			4
#define l_y__count			2
    {	{	"l_y_",
		NULL,				0,
		NULL,				l_y__count,
		X1f4_E4_TEXT_LINK,		l_y__class	},
	line_ymix,							}
#define l_y__reach			(l_pn__reach + l_y__count)
#define l_y__trans			(l_pn__trans + l_y__class)
/* *INDENT-ON* */
#define near_line_class			l_y__trans
#define near_line_count			l_y__reach
#define near_line_shift \
    sizeof(near_line) / sizeof(struct link_type)
}, past_line[] = {
/* *INDENT-OFF* */
#define l_x__class			4
#define l_x__count			2
    {	{	"l_x_",
		NULL,				0,
		NULL,				l_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						l_x__class	},
	line_xmix,							},
#define l_x__reach			(0 + l_x__count)
#define l_x__trans			(0 + l_x__class)
#define lb_x__class			5
#define lb_x__count			1
    {	{	"lb_x_",
		NULL,				0,
		NULL,				lb_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						lb_x__class	},
	line_xpix,							},
#define lb_x__reach			(l_x__reach + lb_x__count)
#define lb_x__trans			(l_x__trans + lb_x__class)
#define lf_x__class			5
#define lf_x__count			1
    {	{	"lf_x_",
		NULL,				0,
		NULL,				lf_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						lf_x__class	},
	line_xsix,							}
#define lf_x__reach			(lb_x__reach + lf_x__count)
#define lf_x__trans			(lb_x__trans + lf_x__class)
/* *INDENT-ON* */
#define past_line_class			lf_x__trans
#define past_line_count			lf_x__reach
#define past_line_shift \
    sizeof(past_line) / sizeof(struct link_type)
}, spin_line[] = {
/* *INDENT-OFF* */
#define l_n__count			2
    {	{	"l_n_",
		_libx1f4i0_lxlist_n_logique,	0,
		NULL,				l_n__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	line_nfix,							}
#define l_n__reach			(0 + l_n__count)
/* *INDENT-ON* */
#define spin_line_class			(spin_line_shift * 4)
#define spin_line_count			l_n__reach
#define spin_line_shift \
    sizeof(spin_line) / sizeof(struct link_type)
};
static const struct x1f4_postmark_type side_k[] = {
    {	MAKE_SINGLE("]", " "),	1				}
};
static const struct x1f4_2ndscope_type side_2[] = {
/* *INDENT-OFF* */
    {	X1f4_E4_POST_MARK,	NULL,
	side_k,			NULL				}
/* *INDENT-ON* */
};
static const struct x1f4_operator_type side_o[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK | X1f4_E4_LEFT_XSET,
				1,
	NULL,			NULL					},
    {	MAKE_SINGLE("[", " "),  NULL,		0200,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK,
				1,
	NULL,			side_2					}
/* *INDENT-ON* */
};

static int
init_list(struct lxlist_type *lxlist_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    if (bits & AUTOLINK_LINK) {
	lxlist_data->link_v = screen_data->lock_v;
    } else {
	lxlist_data->link_v.data = NULL;
	lxlist_data->link_v.free = NULL;
	lxlist_data->link_v.link = NULL;
	lxlist_data->link_v.mode = NULL;
	lxlist_data->link_v.pick = NULL;
	lxlist_data->link_v.slip = NULL;
    }

    if (bits & CODELINK_LINK) {
	lxlist_data->link_w = screen_data->lock_w;
    } else {
	lxlist_data->link_w.data = NULL;
	lxlist_data->link_w.free = _x1f4_e4_free_data;
	lxlist_data->link_w.link = _x1f4_e4_link_data;
	lxlist_data->link_w.mode = _x1f4_e4_mode_data;
    }

    if (bits & EELOOKUP_LINK) {
	lxlist_data->trap_e.eelookup_data = screen_data->lock_g.eelookup_data;
    } else {
	lxlist_data->trap_e.eelookup_data = &_x1f4_type_lookup;
    }

    if (bits & MISSBAIL_LINK) {
	lxlist_data->link_r.call = screen_data->lock_r.call;
	lxlist_data->link_r.fine = screen_data->lock_r.fine;
	lxlist_data->link_r.miss = screen_data->lock_r.miss;
	lxlist_data->link_r.text = screen_data->lock_r.text;
    } else {
	lxlist_data->link_r.call = NULL;
	lxlist_data->link_r.fine = NULL;
	lxlist_data->link_r.miss = NULL;
    }

    if (bits & IMPLICIT_LINK) {
	lxlist_data->link_p.link = screen_data->link_p.link;
	lxlist_data->link_p.text = screen_data->link_p.text;
    } else {
	lxlist_data->link_p.link = NULL;
	lxlist_data->link_p.text = NULL;
    }

    if (bits & COMPOSER_LINK) {
	lxlist_data->link_q.link = screen_data->link_q.link;
	lxlist_data->link_q.text = screen_data->link_q.text;
    } else {
	lxlist_data->link_q.link = NULL;
	lxlist_data->link_q.text = NULL;
    }

    if (bits & RESOURCE_LINK) {
	lxlist_data->link_m = screen_data->lock_m;
    } else {
	lxlist_data->link_m.data = NULL;
	lxlist_data->link_m.free = _x1f4_e4_free_data;
	lxlist_data->link_m.link = _x1f4_e4_link_data;
	lxlist_data->link_m.mode = _x1f4_e4_mode_data;
    }

    if (bits & TEXTFLAT_LINK) {
	lxlist_data->link_e = screen_data->lock_e;
    } else {
	lxlist_data->link_e.data = NULL;
	lxlist_data->link_e.line = NULL;
	lxlist_data->link_e.post = NULL;
	lxlist_data->link_e.push = NULL;
    }

    if (bits & TRANSFER_LINK) {
	lxlist_data->link_o.fine = screen_data->link_o.fine;
    } else {
	lxlist_data->link_o.fine = NULL;
    }

    return link_list(lxlist_data, bits, screen_data);
}


static int
line_data(struct lxlist_type *lxlist_data, unsigned bits,
	  const struct part_type *part_data, void *data, void *text,
	  void **args, void **dana)
{
    struct lxtext_type *lxtext_data;
    const struct lxtype_type *lxtype_data;
    const struct screen_type *screen_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned i;

    screen_data = part_data->screen_data;

    lxtype_data = screen_data->lock_t.data;

    linetext_data = data;
    lxtext_data = text;

    lxtype_data--;

    _x1f4_type_effect(&lxtext_data->lxtype, X1f4_E4_CALL);
    lxtext_data->text = lxlist_data;

    i = screen_data->lock_t.miss;
    for (; i; i--) {
	int code;
	unsigned flags;

	lxtext_data++;

	lxtype_data++;

	line_sail
	    (lxlist_data, screen_data, &linetext_data, lxtext_data,
	     lxtype_data, args, dana);

	flags = lxtype_data->flags;

	if (flags & X1f4_LX_TYPE_EXCEPT) {
	    continue;
	}

	if (1) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, fast_line_shift, fast_line);
	}
	if (bits & AUTOLINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, cast_line_shift, cast_line);
	}

	code = lxtype_data->code;

	if (code == X1f4_E4_BILL || code == X1f4_E4_MODE
	    || code == X1f4_E4_REAL) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, near_line_shift, near_line);
	}

	if (bits & AUTOLINK_LINK
	    || (!lxtype_data->lead && true(code != X1f4_E4_TEXT))) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, past_line_shift, past_line);
	}

	if (flags & LINK_ACCESS) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, long_line_shift, long_line);
	}
	if (flags & POST_ACCESS && bits & AUTOLINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, part_data, &linetext_data, lxtype_data,
		 args, dana, spin_line_shift, spin_line);
	}
    }

    return 0;
}


static int
line_list(struct lxlist_type *lxlist_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    lxlist_data->link_t.datatype[0].context = lxlist_data;

    lxlist_data->link_t.datatype[0].flat = _libx1f4i0_lxlist_flat_list;
    lxlist_data->link_t.datatype[0].lead = _libx1f4i0_lxlist_lead_list;
    lxlist_data->link_t.datatype[0].line = _libx1f4i0_lxlist_line_list;
    lxlist_data->link_t.datatype[0].link = _libx1f4i0_lxlist_link_list;
    lxlist_data->link_t.datatype[0].name = "list";
    lxlist_data->link_t.datatype[0].slip = _libx1f4i0_lxlist_slip_list;
    lxlist_data->link_t.datatype[0].size = 4;
    lxlist_data->link_t.datatype[0].type = screen_data->link_l.type;

    lxlist_data->link_s.nodetype[0].context = lxlist_data;

    _libx1f4i0_lxlist_type_list
	(lxlist_data->link_s.nodetype + 0, screen_data->link_l.type);

    lxlist_data->link_t.datatype[1].context = NULL;

    lxlist_data->link_t.datatype[1].flat = NULL;
    lxlist_data->link_t.datatype[1].lead = NULL;
    lxlist_data->link_t.datatype[1].line = NULL;
    lxlist_data->link_t.datatype[1].link = NULL;
    lxlist_data->link_t.datatype[1].name = NULL;
    lxlist_data->link_t.datatype[1].slip = NULL;
    lxlist_data->link_t.datatype[1].size = 0;
    lxlist_data->link_t.datatype[1].type = 0;

    return 0;
}


static int
line_plus(struct lxlist_type *lxlist_data)
{
    struct cxbend_type *cxbend_data;
    struct cxnear_type *cxnear_data;

    cxbend_data = lxlist_data->rule_f.cxbend;

    cxbend_data[0].bend = 0;
    cxbend_data[0].list = lxlist_data;
    cxbend_data[0].pick = _libx1f4i0_lead_l4list;
    cxbend_data[0].push = _libx1f4i0_head_l4list;
    cxbend_data[0].slip = _libx1f4i0_mind_l4list;

    cxbend_data++;

    cxbend_data[0].bend = 1;
    cxbend_data[0].list = lxlist_data;
    cxbend_data[0].pick = _libx1f4i0_ever_l4list;
    cxbend_data[0].push = _libx1f4i0_tail_l4list;
    cxbend_data[0].slip = _libx1f4i0_slip_l4list;

    cxnear_data = lxlist_data->rule_f.cxnear;

    cxnear_data[0].rule = _libx1f4i0_lxlist_r_drive_b;
    cxnear_data[0].list = lxlist_data;

    cxnear_data++;

    cxnear_data[0].rule = _libx1f4i0_lxlist_r_drive_m;
    cxnear_data[0].list = lxlist_data;

    cxnear_data++;

    cxnear_data[0].rule = _libx1f4i0_lxlist_r_drive_r;
    cxnear_data[0].list = lxlist_data;

    return 0;
}


static int
line_sail(struct lxlist_type *lxlist_data,
	  const struct screen_type *screen_data,
	  struct x1f4_linetext_type **linetext,
	  struct lxtext_type *lxtext_data,
	  const struct lxtype_type *lxtype_data, void **subtext, void **dana)
{
    int code;
    struct x1f4_linetext_type *linetext_data;

    code = lxtype_data->code;

    linetext_data = *linetext;

    lxtext_data->lxtype = *lxtype_data;
    lxtext_data->text = lxlist_data;

    if (0) {
    } else {
	if (code == X1f4_E4_TEXT) {
	    lxtext_data->lxtype.context = &lxlist_data->ever_a;

	    lxtext_data->lxtype.copy = _libx1f4i0_lxnext_copy_text;
	    lxtext_data->lxtype.free = _libx1f4i0_lxnext_free_text;
	    lxtext_data->lxtype.lead = _libx1f4i0_lxnext_lead_text;
	    lxtext_data->lxtype.move = NULL;
	    lxtext_data->lxtype.push = _libx1f4i0_lxnext_push_text;
	    lxtext_data->lxtype.slip = _libx1f4i0_lxnext_slip_text;
	}
    }

    *linetext = linetext_data;

    return 0;
}


static int
line_text(struct lxlist_type *lxlist_data, unsigned bits,
	  const struct part_type *part_data, void *data, void **args,
	  void **dana)
{
#define cxbend_ever			lxlist_data->rule_f.cxbend + 1
#define cxbend_lead			lxlist_data->rule_f.cxbend + 0

    struct x1f4_linetext_type *linetext_data;

    linetext_data = data;

    if (1) {
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, linetext_data, args,
	     sizeof(gray_line) / sizeof(struct line_type), gray_line);
	linetext_data += sizeof(gray_line) / sizeof(struct line_type);
	_libx1f4i0_lxcall_side_line
	    (lxlist_data, bits, part_data, linetext_data, args,
	     sizeof(slip_line) / sizeof(struct line_type), slip_line);
	linetext_data += sizeof(slip_line) / sizeof(struct line_type);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, linetext_data, args,
	     sizeof(used_line) / sizeof(struct line_type), used_line);
	linetext_data += sizeof(used_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxlist_data, bits, part_data, linetext_data, args,
	     sizeof(auto_line) / sizeof(struct line_type), auto_line);
	linetext_data += sizeof(auto_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxlist_data, bits, part_data, linetext_data, args,
	     sizeof(ever_line) / sizeof(struct line_type), ever_line);
	linetext_data += sizeof(ever_line) / sizeof(struct line_type);
	_libx1f4i0_lxcall_side_line
	    (lxlist_data, bits, part_data, linetext_data, args,
	     sizeof(fail_line) / sizeof(struct line_type), fail_line);
	linetext_data += sizeof(fail_line) / sizeof(struct line_type);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, part_data, linetext_data, args,
	     sizeof(head_line) / sizeof(struct line_type), head_line);
	linetext_data += sizeof(head_line) / sizeof(struct line_type);
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, part_data, linetext_data, args,
	     sizeof(tail_line) / sizeof(struct line_type), tail_line);
	linetext_data += sizeof(tail_line) / sizeof(struct line_type);
    }
    if (1) {
	_libx1f4i0_lxcall_zero_line(linetext_data);
    }

    return 0;

#undef cxbend_lead
#undef cxbend_ever
}


static int
link_list(struct lxlist_type *lxlist_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    int status;
    unsigned args_class, args_count, line_class, line_count, main_class,
	main_count, miss, trap;
    void *data;

    args_count = 0;
    line_count = 1;
    main_count = 0;

    pick_miss(&args_class, &line_class, &main_class, bits, screen_data);

    miss = screen_data->lock_t.miss;

    trap = bits & RELATION_LINK ? 1 : 0;

    {
	args_count += gray_line_count;
	line_count += sizeof(gray_line) / sizeof(struct line_type);
	args_count += slip_line_count;
	line_count += sizeof(slip_line) / sizeof(struct line_type);
	args_count += used_line_count;
	line_count += sizeof(used_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	args_count += auto_line_count;
	line_count += sizeof(auto_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	args_count += ever_line_count;
	line_count += sizeof(ever_line) / sizeof(struct line_type);
	args_count += fail_line_count;
	line_count += sizeof(fail_line) / sizeof(struct line_type);
	args_count += head_line_count;
	line_count += sizeof(head_line) / sizeof(struct line_type);
	args_count += tail_line_count;
	line_count += sizeof(tail_line) / sizeof(struct line_type);
    }

    status = lxlist_data->link_w.link
	(lxlist_data->link_w.data, &data,
	 sizeof(struct lxtext_type) * (1 + miss) + fm_size(away)
	 + (args_class + args_count) * sizeof(int)
	 + (line_class + line_count) * sizeof(struct x1f4_linetext_type)
	 + (sizeof(struct tide_type) + sizeof(struct x1f4_operator_type *))
	 * fb_size() + sizeof(struct x1f4_operator_type *)
	 + ((sizeof(struct x1f4_ftaa_type) + sizeof(struct x1f4_ftca_type))
	    * (miss + 1) + sizeof(struct x1f4_fpdetail_type)) * trap
	 + main_class + main_count);
    if (status) {
	status = LINK_ERROR;
    } else {
	struct part_type part;
	struct x1f4_fpdetail_type *fpdetail_data;
	void *args, *dana, *side, *tile, *text;

	tile = (struct x1f4_linetext_type *) data + line_class;

	if (trap) {
	    fpdetail_data = (void *)
		((struct x1f4_linetext_type *) tile + line_count);
	    part.ftaa_data = (void *) (fpdetail_data + 1);
	    part.ftca_data = (void *) (part.ftaa_data + miss + 1);
	    fpdetail_data->bits1 = X1f4_E4_TYPE_LIST | X1f4_E4_XSET_LIST;
	    fpdetail_data->ftaa_data = part.ftaa_data;
	    fpdetail_data->ftca_data = part.ftca_data;
	    dana = part.ftca_data + miss + 1;
	} else {
	    fpdetail_data = NULL;
	    part.ftaa_data = NULL;
	    part.ftca_data = NULL;
	    dana = (struct x1f4_linetext_type *) tile + line_count;
	}

	/*
	 * lxmiss and lxtext records required together
	 */
	text = (struct lxmiss_type *) dana + 1;
	side = (struct lxtext_type *) text + miss + 1;
	side = (struct tide_type *) side + fb_size();
	args = (struct x1f4_operator_type **) side + fb_size() + 1;
	dana = (int *) args + args_class + args_count;

	lxlist_data->link_f.data = data;
	lxlist_data->link_f.side = side;
	lxlist_data->link_f.text = text;

	((struct lxmiss_type *) text - 1)->data.miss = miss;

	part.screen_data = screen_data;

        line_plus(lxlist_data);

	line_data(lxlist_data, bits, &part, data, text, &args, &dana);
	line_text(lxlist_data, bits, &part, tile, &args, &dana);
	_libx1f4i0_lxcall_line_side(side, data, side_o, side_f, fb_size());

	line_list(lxlist_data, bits, screen_data);

	if (trap) {
	    part.ftaa_data->linetext_data = NULL;
	    part.ftaa_data->type = X1f4_E4_VOID;
	    part.ftca_data->linetext_data = NULL;
	    part.ftca_data->type = X1f4_E4_VOID;

	    x1f4_post_mxpath
		(screen_data->link_y.linetrap_data->fpaccess, fl_queryf, 7,
		 fpdetail_data);
	}
    }

    return status;
}


static int
pick_miss(unsigned *args, unsigned *line, unsigned *pick, unsigned bits,
	  const struct screen_type *screen_data)
{
    const struct lxtype_type *lxtype_data;
    unsigned args_count = 0, line_count = 0, i, main_count = 0;

    lxtype_data = screen_data->lock_t.data;
    lxtype_data--;

    i = screen_data->lock_t.miss;
    for (; i; i--) {
	int code;
	unsigned deck, flags;

	lxtype_data++;

	flags = lxtype_data->flags;

	if (flags & X1f4_LX_TYPE_EXCEPT) {
	    continue;
	}

	deck = lxtype_data->size + 1;

	if (1) {
	    args_count += fast_line_count;
	    line_count += fast_line_shift;
	    main_count += deck * fast_line_shift + fast_line_class;
	}
	if (bits & AUTOLINK_LINK) {
	    args_count += cast_line_count;
	    line_count += cast_line_shift;
	    main_count += deck * cast_line_shift + cast_line_class;
	}

	if (flags & FLAT_REFLEX) {
	} else {
	    args_count += l_e_type_post;
	    args_count += l_g_type_post;
	    args_count += lbe_type_post;
	    args_count += lbg_type_post;
	    args_count += lfe_type_post;
	    args_count += lfg_type_post;
	}
	if (flags & LINK_ACCESS) {
	    args_count += long_line_count;
	    line_count += long_line_shift;
	    main_count += deck * long_line_shift + long_line_class;
	}
	if (flags & POST_ACCESS && bits & AUTOLINK_LINK) {
	    args_count += spin_line_count;
	    line_count += spin_line_shift;
	    main_count += deck * spin_line_shift + spin_line_class;
	}

	code = lxtype_data->code;

	if (code == X1f4_E4_BILL || code == X1f4_E4_MODE
	    || code == X1f4_E4_REAL) {
	    args_count += near_line_count;
	    line_count += near_line_shift;
	    main_count += deck * near_line_shift + near_line_class;
	    if (flags & FLAT_REFLEX) {
	    } else {
		args_count += l_i_type_post;
	    }
	}

	if (bits & AUTOLINK_LINK
	    || (!lxtype_data->lead && true(code != X1f4_E4_TEXT))) {
	    args_count += past_line_count;
	    line_count += past_line_shift;
	    main_count += deck * past_line_shift + past_line_class;
	}
    }

    *args = args_count;
    *line = line_count;
    *pick = main_count;

    return 0;
}


static void
bo_slmy00(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;
    *line++ = 0;

    *args = line;
}


static void
born_lmlm(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
borne_lmc(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = X1f4_E4_CASE;

    *args = line;
}


static void
borne_lmm(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
borne_lmn(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = nodetype_data->code;

    *args = line;
}


static void
borneo__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;
}


static void
borneo_cl(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_CASE;
    *line++ = screen(screen)->link_l.type;

    *args = line;
}


static void
borneo_l_(u_case_args_____1)
{
    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
borneo_lc(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_CASE;

    *args = line;
}


static void
borneo_ll(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;

    *args = line;
}


static void
borneo_lm(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
borneo_ln(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = nodetype_data->code;

    *args = line;
}


static void
fine_pmix(v_case_args_____1)
{
    int *line, near, type;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;
    *line++ = X1f4_E4_MODE;
    *line++ = type;

    *args = line;

    if (type == X1f4_E4_REAL) {
	near = 2;
    } else {
	if (type == X1f4_E4_MODE) {
	    near = 1;
	} else {
	    if (true(type == X1f4_E4_BILL)) {
		near = 0;
	    } else {
	    }
	}
    }

    linetext_data->context =
	((struct lxlist_type *)
	 ((struct lxtext_type *) linetext_data->context)->text)->rule_f.cxnear
	+ near;
}


static void
line_emix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_emix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE, ~0,
	 rate_emix);
}


static void
line_epix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_esix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_epix);
}


static void
line_esix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_esix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_esix);
}


static void
line_gmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 &part(screen)->ftaa_data, rate_gmix);
}


static void
line_gpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_gpix);
}


static void
line_gsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_gsix);
}


static void
line_imix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_imix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 &part(screen)->ftaa_data, rate_imix);
}


static void
line_nfix(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;

    *args = line;

    linetext_data->function.type = nodetype_data->code;
}


static void
line_nset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
line_pmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_pmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 rate_pmix);
}


static void
line_ppix(v_case_args_____1)
{
    int *line, type;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = type;

    *args = line;

    if (nodetype_data->copy || may_include(nodetype_data)) {
	linetext_data->function.function = _libx1f4i0_lxlist_pbclass_v;
    } else {
	if (X1f4_E4_LAST < type) {
	    linetext_data->function.function = _libx1f4i0_lxlist_pbclass_u;
	} else {
	    if (type == X1f4_E4_TEXT) {
		linetext_data->function.function = _libx1f4i0_lxlist_pbclass_t;
	    } else {
		linetext_data->context =
		    ((struct lxlist_type *)
		     ((struct lxtext_type *)
		      linetext_data->context)->text)->rule_f.cxbend + 1;

		if (type == X1f4_E4_REAL) {
		    linetext_data->function.function =
			_libx1f4i0_lxlist_p0class_r;
		} else {
		    if (type == X1f4_E4_MODE) {
			linetext_data->function.function =
			    _libx1f4i0_lxlist_p0class_m;
		    } else {
			if (type == X1f4_E4_BILL) {
			    linetext_data->function.function =
				_libx1f4i0_lxlist_p0class_b;
			} else {
			}
		    }
		}
	    }
	}
    }
}


static void
line_psix(v_case_args_____1)
{
    int *line, type;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = type;

    *args = line;

    if (nodetype_data->copy || may_include(nodetype_data)) {
	linetext_data->function.function = _libx1f4i0_lxlist_pfclass_v;
    } else {
	if (X1f4_E4_LAST < type) {
	    linetext_data->function.function = _libx1f4i0_lxlist_pfclass_u;
	} else {
	    if (type == X1f4_E4_TEXT) {
		linetext_data->function.function = _libx1f4i0_lxlist_pfclass_t;
	    } else {
		linetext_data->context =
		    ((struct lxlist_type *)
		     ((struct lxtext_type *)
		      linetext_data->context)->text)->rule_f.cxbend + 0;

		if (type == X1f4_E4_REAL) {
		    linetext_data->function.function =
			_libx1f4i0_lxlist_p0class_r;
		} else {
		    if (type == X1f4_E4_MODE) {
			linetext_data->function.function =
			    _libx1f4i0_lxlist_p0class_m;
		    } else {
			if (type == X1f4_E4_BILL) {
			    linetext_data->function.function =
				_libx1f4i0_lxlist_p0class_b;
			} else {
			}
		    }
		}
	    }
	}
    }
}


static void
line_qmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 &part(screen)->ftca_data, rate_qmix);
}


static void
line_qpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_qpix);
}


static void
line_qsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_qsix);
}


static void
line_rmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_pmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 rate_rmix);
}


static void
line_smap(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_MODE;

    *args = line;
}


static void
line_sset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
line_xmix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xmix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 rate_xmix);
}


static void
line_xpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_xpix);
}


static void
line_xsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxlist_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_xsix);
}


static void
line_ymix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_ymix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 &part(screen)->ftca_data, rate_ymix);
}


int
x1f4_init_lxlist(void **lxlist, unsigned bits,
		 const struct screen_type *screen_data)
{
    int (*link) (void *, void **, unsigned), status;
    void *list, *text;

    if (bits & CODELINK_LINK) {
	link = screen_data->lock_w.link;
	text = screen_data->lock_w.data;
    } else {
	link = _x1f4_e4_link_data;
	text = (void *) 0;
    }

    status = link(text, &list, sizeof(struct lxlist_type));
    if (status) {
	status = LINK_ERROR;
    } else {
	status = init_list(list, bits, screen_data);
	if (status) {
	    if (bits & CODELINK_LINK) {
		screen_data->lock_w.free(text, list);
	    } else {
		_x1f4_e4_free_data(text, list);
	    }
	} else {
	    _libx1f4i0_seed_rand();

	    *lxlist = list;
	}
    }

    return status;
}
