/*
 * lxlist-a.l.c
 * Copyright (C) 2008-2014, 2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlead.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define __HIGH_CLEAR__			1
#define __LIME_CLEAR__			2
#define __MISS_CLEAR__			3

#define __HERE_CLEAR__			__LIME_CLEAR__

#define dxlist(list) \
    ((struct dxlist_type *) (list))
#define lxlist(list) \
    ((struct lxlist_type *) (list))

#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define byte(miss)			((unsigned char *) (miss))

#if __HERE_CLEAR__ == __LIME_CLEAR__
typedef struct null_type {
    int status;
    struct excase_type *excase_data;
    struct lxtext_type *lxtext_data;
    void *dxlist;
} null_type;
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

#if __HERE_CLEAR__ == __LIME_CLEAR__
static int null_data(void *, void *);
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

#if __HERE_CLEAR__ == __LIME_CLEAR__
static int
null_data(void *null, void *data)
{
    unsigned link;

    link = byte(data)[4] << 030 | byte(data)[5] << 020 | byte(data)[6] << 010
	| byte(data)[7];
    if (~link) {
	int excess, (*free)(void *, void *, struct excase_type **,
			    const struct excase_type *);
	struct lxtext_type *lxtext_link;
	struct null_type *null_data;

	null_data = null;

	lxtext_link = null_data->lxtext_data + link;

	if (lxtext_link->lxtype.flags & X1f4_LX_PICK_ACCESS) {
	    free = lxtext_link->lxtype.pick;
	} else {
	    free = lxtext_link->lxtype.free;
	}
	if (free) {
	    excess = free
		(lxtext_link->lxtype.context,
		 I_USER(byte(data) + 8), &null_data->excase_data,
		 lock_excase(null_data->dxlist));
	    if (excess) {
		null_data->status =
		    _libx1f4i0_lxcall_land_slip(excess, null_data->status);
	    }
	}
    }

    return 0;
}
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */


int
_libx1f4i0_lxlist_null_list(void *lxlist,
			    struct excase_type **excase, void *dxlist)
{
    int status = 0;
    unsigned size;

    size = dxlist(dxlist)->size;
    if (size) {
	int excess;
	struct excase_type *excase_data;
#if __HERE_CLEAR__ != __LIME_CLEAR__
	struct lxtext_type *lxtext_data;
#endif				/* __HERE_CLEAR__ != __LIME_CLEAR__ */
#if __HERE_CLEAR__ == __LIME_CLEAR__
	struct null_type null;
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */
	void *list;

#if __HERE_CLEAR__ == __LIME_CLEAR__
	null.lxtext_data = lxlist(lxlist)->link_f.text;
	null.status = 0;
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

#if __HERE_CLEAR__ != __LIME_CLEAR__
	lxtext_data = lxlist(lxlist)->link_f.text;
#endif				/* __HERE_CLEAR__ != __LIME_CLEAR__ */

	if (excase) {
	    excase_data = *excase;
	} else {
	    excase_data = NULL;
	}

#if __HERE_CLEAR__ == __LIME_CLEAR__
	null.dxlist = dxlist;
	null.excase_data = excase_data;
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

	list = dxlist(dxlist)->list;

#if __HERE_CLEAR__ == __LIME_CLEAR__
	_libx1f4i0_lime_l4list(list, &null, null_data);
#else
	while (size) {
	    unsigned link;
	    void *data;

	    size--;

# if __HERE_CLEAR__ == __MISS_CLEAR__
	    _libx1f4i0_peek_l4list(list, 0, &data);
# else
	    _libx1f4i0_peek_l4list(list, size, &data);
# endif				/* __HERE_CLEAR__ == __MISS_CLEAR__ */

	    link = byte(data)[4] << 030 | byte(data)[5] << 020
		| byte(data)[6] << 010 | byte(data)[7];
	    if (~link) {
		int (*free)(void *, void *, struct excase_type **,
			    const struct excase_type *);
		struct lxtext_type *lxtext_link;

		lxtext_link = lxtext_data + link;

		if (lxtext_link->lxtype.flags & X1f4_LX_PICK_ACCESS) {
		    free = lxtext_link->lxtype.pick;
		} else {
		    free = lxtext_link->lxtype.free;
		}
		if (free) {
		    excess = free
			(lxtext_link->lxtype.context,
			 I_USER(byte(data) + 8), &excase_data,
			 lock_excase(dxlist));
		    if (excess) {
			status = _libx1f4i0_lxcall_land_slip(excess, status);
		    }
		}
	    }

# if __HERE_CLEAR__ == __MISS_CLEAR__
	    if (1) {
		excess = _libx1f4i0_miss_l4list(list, 0);
		if (excess) {
		    status = _libx1f4i0_lxlist_stat_list
			(lxlist, status, excess);
		}
	    }
# endif				/* __HERE_CLEAR__ == __MISS_CLEAR__ */
	}
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

#if __HERE_CLEAR__ == __LIME_CLEAR__
	excase_data = null.excase_data;
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

#if __HERE_CLEAR__ == __LIME_CLEAR__
	status = null.status;
#endif				/* __HERE_CLEAR__ == __LIME_CLEAR__ */

#if __HERE_CLEAR__ == __HIGH_CLEAR__ || __HERE_CLEAR__ == __LIME_CLEAR__
	if (1) {
	    excess = _libx1f4i0_high_l4list(list);
	    if (excess) {
		status = _libx1f4i0_lxlist_stat_list(lxlist, status, excess);
	    }
	}
#endif				/* __HERE_CLEAR__ == __HIGH_CLEAR__ || __... */

	if (excase) {
	    *excase = excase_data;
	} else {
	    if (excase_data) {
		excess = x1f4_fail_lxlead(excase_data);
		if (excess) {
		    status = _libx1f4i0_lxcall_land_slip(excess, status);
		}
	    }
	}

#if __HERE_CLEAR__ == __HIGH_CLEAR__ || __HERE_CLEAR__ == __LIME_CLEAR__
	dxlist(dxlist)->size = 0;
#endif				/* __HERE_CLEAR__ == __HIGH_CLEAR__ || __... */

#if __HERE_CLEAR__ == __MISS_CLEAR__
	dxlist(dxlist)->size = size;
#endif				/* __HERE_CLEAR__ == __MISS_CLEAR__ */
    }

    return status;
}
