/*
 * lxlist-p.x.c
 * Copyright (C) 2011-2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <ct.h>
#include <fd.h>
#include <it.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define I_LIST(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define byte(miss)			((unsigned char *) (miss))

extern const void *const x1f4_a1_walk_link;

static int ever_call(void **, void **, unsigned, struct dxlist_type *, char *,
		     int *, const int *, unsigned, unsigned);
static int rule_list(void *, void *);
static int type_list(void *, void *);

static int
ever_call(void **post_data, void **line_data, unsigned parts,
	  struct dxlist_type *dxlist_data, char *used, int *post_args,
	  const int *line_args, unsigned line, unsigned link)
{
    memcpy(post_data, line_data + 2, line * sizeof(void *));

    post_data += line;

    _libx1f4i0_lime_l4list(dxlist_data->list, &post_data, rule_list);

    memcpy(post_args, line_args + 2, line * sizeof(int));
    post_args += line;
    _libx1f4i0_lime_l4list(dxlist_data->list, &post_args, type_list);
    if (parts & X1f4_E4_POST_TYPE) {
	memcpy(post_args, line_args + 4 + line, line * sizeof(int));
	post_args += line;
	if (1) {
	} else {
	    memset(post_args, 0, link * sizeof(int));
	}
    }

    return 0;
}


static int
rule_list(void *line, void *data)
{
    void ***rule;

    rule = line;
    **rule = byte(data) + 8;
    (*rule)++;

    return 0;
}


static int
type_list(void *line, void *data)
{
    int **args;

    args = line;
    **args = effect_type(data);
    (*args)++;

    return 0;
}


int
_libx1f4i0_lxlist_x_pending(void *context, void *output, void **input)
{
    int status;
    struct lxlist_type *lxlist_data;
    struct x1f4_dxcast_type *dxcast_data;
    void **track, **used;

    if (output == x1f4_a1_walk_link) {
	track = input[1];
    } else {
	track = input;
    }

    used = track[-1];

    lxlist_data = context;

    dxcast_data = I_USER(track[0]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	struct x1f4_linetext_type *linetext_data;

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	    struct dxlist_type *dxlist_data;
	    struct x1f4_function_type function;
	    const struct x1f4_function_type *function_text;
	    unsigned class, count, parts, scale = 0, trip;
	    void *beta;

	    dxlist_data = I_LIST(track[1]);

	    count = dxlist_data->size;

	    function_text = used[X1f4_E4_RENDITION];

	    class = function_text->count - 2;

	    function.count = count + class;

	    parts = function_text->flags;

	    function.flags = parts & X1f4_E4_POST_TYPE;

	    if (function.flags) {
		scale = 1;
	    }

	    trip = (sizeof(void *) + (sizeof(int) << scale)) * function.count
		+ sizeof(void *);

	    status = lxlist_data->link_v.link
		(lxlist_data->link_v.data, &beta, trip);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_link(lxlist_data);
	    } else {
		void *clear[3], **frame;

		frame = beta;
		frame++;

		function.args = (void *) (frame + function.count);

		frame[-1] = used;

		ever_call
		    (frame, track, parts, dxlist_data, track[-1],
		     (int *) function.args, function_text->args, class, count);

		if (output == x1f4_a1_walk_link) {
		    clear[0] = input[0];
		    clear[1] = frame;
		    clear[2] = input[2];

		    frame = clear;
		}

		status = _libx1f4i0_lxcall_post_call
		    (linetext_data, &function, output, 0, frame,
		     &lxlist_data->link_v, &lxlist_data->link_p,
		     &lxlist_data->link_q, &lxlist_data->link_e,
		     lxlist_data->trap_e.eelookup_data);
		if (status) {
		} else {
		    if (output == x1f4_a1_walk_link) {
			input[2] = clear[2];
		    }
		}
	    }
	} else {
	    status = _libx1f4i0_lxlist_stat_zero(lxlist_data, "l_xcall", 7);
	}
    } else {
	status = _libx1f4i0_lxlist_stat_roll(lxlist_data, "l_xcall", 7);
    }

    return status;
}
