/*
 * lxroll-f.c.c
 * Copyright (C) 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <netdb.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>
#include <netinet/tcp.h>

#include <sys/socket.h>
#include <sys/types.h>

#include <e4.h>
#include <lxfile.h>
#include <lxroll-defs.h>
#include <lxroll-inter.h>
#include <lxroll-types.h>

#define __USE_GETADDRINFO__		1

#define I_FILE(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

static int case_inet(void *, const char *, int, int);

static int
case_inet(void *lxroll, const char *node, int sort, int connection)
{
    int status;
#if __USE_GETADDRINFO__
    struct addrinfo *addresses;
#else
    struct hostent *hostinfo;
    struct sockaddr_in address;
#endif				/* __USE_GETADDRINFO__ */

#if __USE_GETADDRINFO__
    status = getaddrinfo(node, NULL, NULL, &addresses);
    if (status) {
	status = _libx1f4i0_lxroll_stat_info
	    (lxroll, node, gai_strerror(status));
    } else {
	struct addrinfo *info;
	unsigned matched = 0, port;

	port = htons(sort);

	info = addresses;
	while (info) {
	    if (info->ai_family == AF_INET
		&& (!info->ai_socktype || info->ai_socktype == SOCK_STREAM)) {
		struct sockaddr_in *address;

		address = (void *) info->ai_addr;

		address->sin_port = port;

		matched++;

		/* socket(info->ai_family, info->ai_socktype,
			  info->ai_protocol)?  guess not. */

		status = connect(connection, info->ai_addr, info->ai_addrlen);
		if (status) {
		    status = _libx1f4i0_lxroll_stat_call(lxroll, "connect");
		} else {
		    break;
		}
	    }

	    info = info->ai_next;
	}

	if (matched) {
	} else {
	    status = _libx1f4i0_lxroll_stat_lock(lxroll, node);
	}

	freeaddrinfo(addresses);
    }
#else
    address.sin_family = AF_INET;
    address.sin_port = htons(sort);

    hostinfo = gethostbyname(node);
    if (hostinfo) {
	address.sin_addr = *(struct in_addr *) hostinfo->h_addr;
	status = connect
	    (connection, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll, "connect");
	}
    } else {
	status = _libx1f4i0_lxroll_stat_info(lxroll, node, hstrerror(h_errno));
    }
#endif				/* __USE_GETADDRINFO__ */

    return status;
}


int
_libx1f4i0_lxroll_c_forward(void *context, void *output, void **input)
{
    int fd = -1, ld = -1, status;

    do {
        struct lxroll_type *lxroll_data;
        struct x1f4_dxfile_type *dxfile_data, *dxfile_text;
	unsigned bits;
	void *lxfile;

        lxroll_data = context;

	lxfile = lxroll_data->link_l.file;

        dxfile_data = I_FILE(input[0]);

        dxfile_text = I_FILE(input[1]);

	if (dxfile_data == dxfile_text) {
	    status = _libx1f4i0_lxroll_stat_back(lxroll_data);
	    if (1) {
		break;
	    }
	}

	/*
	 * connect(2) before closing any files.
	 */

	fd = socket(PF_INET, SOCK_STREAM, 0);
	if (fd == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "socket");
            if (1) {
                break;
            }
	}

	bits = I_MODE(input[4]);

	if (bits & TCP_NO_DELAY) {
	    int part = 1;

	    status = setsockopt
		(fd, IPPROTO_TCP, TCP_NODELAY, &part, sizeof(int));
	    if (status) {
		status = _libx1f4i0_lxroll_stat_part
		    (lxroll_data, TCP_NO_DELAY);
		if (1) {
		    break;
		}
	    }
	}

	status = case_inet
	    (lxroll_data, I_TEXT(input[2]), I_MODE(input[3]), fd);
	if (status) {
	    break;
	}

	ld = dup(fd);
	if (ld == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "dup");
            if (1) {
                break;
            }
	}

	status = _libx1f4i0_lxroll_land_roll
	    (lxroll_data, lxfile, bits, dxfile_data, &fd, dxfile_text, &ld);
    } while (0);

    if (fd ^ -1) {
	close(fd);
    }

    if (ld ^ -1) {
	close(ld);
    }

    return status;
}
